#include "intpk.h"

#define NPROP 43
#define NRIMAX 4
#define DELTAT 0.0001 /* used to bias the temperaure at he saturation line so that the \
                          property value is calculated for the proper phase  */

/*
    Because arrays in C start at index '0' instead of '1', the dimension of the following
    arrays has been increased by 1 so that the original index values can be used in the
    converted coding.  Nothing is stored in the [0] locations of the arrays.
*/
double PROPR[NPROP + 1];
double PROPSI[NPROP + 1];
double RI[NRIMAX + 1];
double WAVRI[NRIMAX + 1];

int IPCHK[6];
int IPFLG[6];
int IWANT[NPROP + 1];
int IWORK[NPROP + 1];
int IRIFLG[NRIMAX + 1];

int IERR, I2PH, ISFLG, ICFLG; /* these don't need to be external variables, but this is
                                 easier than declaring them in every function.  */

/*
    Function declaration from file "auxpk.c" that are used in NIST.
*/
void IVZERO(int IVEC[], int ILEN);

/*
    Function declarations:
*/
double SATTEMP(double PMPA);
double GETPROP(double PMPA, double TDEGK, int IPROP);

/*
!  GENERIC INTERFACE FOR A LIQUID PROPERTY AS A FN. OF PRESSURE (MPA) & TEMPERATURE (K).
*/
double GETLIQPROP(double PMPA, double T, int IPROP)
{
    double getLiqProp;
    double TDEGK, TLIQ;

    TLIQ = SATTEMP(PMPA) - DELTAT;
    TDEGK = (T < TLIQ) ? T : TLIQ;
    getLiqProp = GETPROP(PMPA, TDEGK, IPROP);

    return getLiqProp;
}

/*
!  GENERIC INTERFACE FOR A PROPERTY AS A FN. OF PRESSURE (MPA) & TEMPERATURE (K).
*/
double GETPROP(double PMPA, double TDEGK, int IPROP)
{
    double RHO, DPD;

    IERR = DENS0(&RHO, PMPA, TDEGK, &DPD, IWORK, PROPR);

    IVZERO(IWANT, NPROP + 1);
    IVZERO(IPCHK, 6);

    IWANT[IPROP] = 1;

    PROPS(IWANT, TDEGK, RHO, PROPSI, PROPR, 0, &I2PH, 0, &ISFLG, 0, &ICFLG, IPCHK, IPFLG, 0, 0,
          WAVRI, RI, IRIFLG);

    return PROPSI[IPROP];
}

/*
!  GENERIC INTERFACE FOR A VAPOR PROPERTY AS A FN. OF PRESSURE (MPA) & TEMPERATURE (K).
*/
double GETVAPPROP(double PMPA, double T, int IPROP)
{
    double getVapProp;
    double TDEGK, TVAP;

    TVAP = SATTEMP(PMPA) + DELTAT;
    TDEGK = (T > TVAP) ? T : TVAP;
    getVapProp = GETPROP(PMPA, TDEGK, IPROP);

    return getVapProp;
}

/*
!  SATURATED LIQUID ENTHALPY (KJ/KG) AS A FN. OF PRESSURE (MPA).
*/
double HF(double PMPA)
{
    double TDEGK, RHOLIQ, RHOVAP;

    IERR = TSAT(PMPA, &TDEGK, &RHOLIQ, &RHOVAP, IWORK, PROPR);

    IVZERO(IWANT, NPROP + 1);
    IVZERO(IPCHK, 6);

    IWANT[6] = 1;

    PROPS(IWANT, TDEGK, RHOLIQ, PROPSI, PROPR, 0, &I2PH, 0, &ISFLG, 0, &ICFLG, IPCHK, IPFLG, 0, 0,
          WAVRI, RI, IRIFLG);

    return PROPSI[6];
}

/*
!  LATENT HEAT (KJ/KG) AS A FN. OF PRESSURE (MPA).
*/
double HFG(double PMPA)
{
    double TDEGK, RHOLIQ, RHOVAP, HF, HG;

    IERR = TSAT(PMPA, &TDEGK, &RHOLIQ, &RHOVAP, IWORK, PROPR);

    IVZERO(IWANT, NPROP + 1);
    IVZERO(IPCHK, 6);

    IWANT[6] = 1;

    PROPS(IWANT, TDEGK, RHOLIQ, PROPSI, PROPR, 0, &I2PH, 0, &ISFLG, 0, &ICFLG, IPCHK, IPFLG, 0, 0,
          WAVRI, RI, IRIFLG);
    HF = PROPSI[6];

    PROPS(IWANT, TDEGK, RHOVAP, PROPSI, PROPR, 0, &I2PH, 0, &ISFLG, 0, &ICFLG, IPCHK, IPFLG, 0, 0,
          WAVRI, RI, IRIFLG);
    HG = PROPSI[6];

    return (HG - HF);
}

/*
!  SATURATED VAPOR ENTHALPY (KJ/KG) AS A FN. OF PRESSURE (MPA).
*/
double HG(double PMPA)
{
    double TDEGK, RHOLIQ, RHOVAP, HG;

    IERR = TSAT(PMPA, &TDEGK, &RHOLIQ, &RHOVAP, IWORK, PROPR);

    IVZERO(IWANT, NPROP + 1);
    IVZERO(IPCHK, 6);

    IWANT[6] = 1;

    PROPS(IWANT, TDEGK, RHOVAP, PROPSI, PROPR, 0, &I2PH, 0, &ISFLG, 0, &ICFLG, IPCHK, IPFLG, 0, 0,
          WAVRI, RI, IRIFLG);

    return PROPSI[6];
}

/*
!  INTERFACE ROUTINE FOR CALCULATION OF PRANDTL NO.
!
!     IPROP =  1  ;  SUBCOOLED LIQUID
!              2  :  SATURATED LIQUID
!              3  :  SATURATED VAPOR
!              4  :  SUPERHEATED VAPOR
!
!     NOTE: IF SATURATED DENSITY IS REQUESTED, TDEGK IS IGNORED.
*/
double PR(double PMPA, double TDEGK, int IPROP)
{
    double TF, CP, VISC, COND;

    TF = SATTEMP(PMPA);

    if (IPROP == 2)
        TDEGK = TF - DELTAT;

    else if (IPROP == 3)
        TDEGK = TF + DELTAT;

    else if (IPROP == 1)
        TDEGK = (TDEGK < TF - DELTAT) ? TDEGK : (TF - DELTAT);

    else
        TDEGK = (TDEGK > TF + DELTAT) ? TDEGK : (TF + DELTAT);

    CP = 1000. * GETPROP(PMPA, TDEGK, 9);
    VISC = GETPROP(PMPA, TDEGK, 21);
    COND = GETPROP(PMPA, TDEGK, 20);

    return (CP * VISC / COND);
}

/*
!  INTERFACE ROUTINE FOR CALCULATION OF PHASIC DENSITY.
!
!     IPROP =  1  ;  SUBCOOLED LIQUID
!              2  :  SATURATED LIQUID
!              3  :  SATURATED VAPOR
!              4  :  SUPERHEATED VAPOR
!
!     NOTE: IF SATURATED DENSITY IS REQUESTED, TDEGK IS IGNORED.
*/
double RHO(double PMPA, double TDEGK, int IPROP)
{
    double TF, RHOLIQ, RHOVAP, RHO1, DPD;

    IERR = TSAT(PMPA, &TF, &RHOLIQ, &RHOVAP, IWORK, PROPR);

    if (IPROP == 2)
        return RHOLIQ;

    else if (IPROP == 3)
        return RHOVAP;

    else if (IPROP == 1)
        TDEGK = (TDEGK < TF - DELTAT) ? TDEGK : (TF - DELTAT);

    else
        TDEGK = (TDEGK > TF + DELTAT) ? TDEGK : (TF + DELTAT);

    IERR = DENS0(&RHO1, PMPA, TDEGK, &DPD, IWORK, PROPR);

    return RHO1;
}

/*
!  SATURATION PRESSURE (MPA) AS A FN. OF TEMPERATURE (K).
*/
double SATP(double TDEGK)
{
    double PMPA, RHOLIQ, RHOVAP;

    IERR = PSAT(TDEGK, &PMPA, &RHOLIQ, &RHOVAP, IWORK, PROPR);

    return PMPA;
}

/*
!  SATURATION TEMPERATURE (K) AS A FN. OF PRESSURE (MPA).
*/
double SATTEMP(double PMPA)
{
    double TF, RHOLIQ, RHOVAP, RHO1, DPD;

    IERR = TSAT(PMPA, &TF, &RHOLIQ, &RHOVAP, IWORK, PROPR);

    return TF;
}

/*
!  SURFACE TENSION (N/M) AS A FN. OF PRESSURE (MPA).
*/
double SURFT(double PMPA)
{
    double TF, SIGMA;

    TF = SATTEMP(PMPA);
    IERR = SURF(TF, &SIGMA);

    return SIGMA;
}
