#include <math.h>

double DLSAT(double TR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** DLSAT ***                                    C
C THIS ROUTINE ESTIMATES THE REDUCED SAT. LIQUID DENSITY AT A GIVEN    C
C REDUCED TEMPERATURE T/TC USING THE EQUATION OF SAUL AND WAGNER (J.   C
C PHYS. CHEM. REF. DATA, 16, 893 (1987) [AS MODIFIED TO 1990 T-SCALE   C
C BY WAGNER AND PRUSS, J. PHYS. CHEM. REF. DATA, 22, 783 (1993)])      C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TR      R    I   REDUCED TEMPERATURE T/TC                            C
C DLSAT   R    O   REDUCED SATURATED LIQUID DENSITY RHO/RHOC           C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 17JUL95 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
     
	static const double E1 = 1.0/3.0;
    static const double E2 = 2.0/3.0;
    static const double E3 = 5.0/3.0;
    static const double E4 = 16.0/3.0; 
    static const double E5 = 43.0/3.0; 
    static const double E6 = 110.0/3.0; 
 
 /* 
 	B[0] is set to zero, this element is not used.  It is simply included so that the original
    FORTRAN subscripts can be used.  This is because all arrays in C start from the 
    index [0] as opposed to FORTRAN that starts at (1).
*/    
    static const double B[] = { 0.0, 1.99274064, 1.09965342, -0.510839303, -1.75493479,
                                -45.5170352, -6.7469445e5 };
    double TAU;
    double dlsat;
     
    TAU = 1.0 - TR;
    dlsat = 1.0 + B[1]*pow(TAU, E1) + B[2]*pow(TAU, E2) + B[3]*pow(TAU, E3) 
                + B[4]*pow(TAU, E4) + B[5]*pow(TAU, E5) + B[6]*pow(TAU, E6);
    return dlsat;
}


double DVSAT(double TR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** DVSAT ***                                    C
C THIS ROUTINE ESTIMATES THE REDUCED SAT. VAPOR DENSITY AT A GIVEN     C
C REDUCED TEMPERATURE T/TC USING THE EQUATION OF SAUL AND WAGNER (J.   C
C PHYS. CHEM. REF. DATA, 16, 893 (1987) [AS MODIFIED TO 1990 T-SCALE   C
C BY WAGNER AND PRUSS, J. PHYS. CHEM. REF. DATA, 22, 783 (1993)])      C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TR      R    I   REDUCED TEMPERATURE T/TC                            C
C DVSAT   R    O   REDUCED SATURATED VAPOR DENSITY RHO/RHOC            C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 17JUL95 - INITIAL CREATION BY AHH                                    C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/     
    static const double E1 = 1.0/3.0; 
    static const double E2 = 2.0/3.0; 
    static const double E3 = 4.0/3.0;
    static const double E4 = 3.0; 
    static const double E5 = 37.0/6.0; 
    static const double E6 = 71.0/6.0;
      
    static const double C[] = { 0.0, -2.03150240, -2.68302940, -5.38626492, 
                                -17.2991605, -44.7586581, -63.9201063 };
    double TAU;
    double dvsat;
    
    TAU = 1.0 - TR;
    dvsat =  C[1]*pow(TAU, E1) + C[2]*pow(TAU, E2) + C[3]*pow(TAU, E3) 
          +  C[4]*pow(TAU, E4) + C[5]*pow(TAU, E5) + C[6]*pow(TAU, E6);
    dvsat = exp(dvsat);
    
    return dvsat;
}


double DLSATC(double TR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** DLSATC ***                                   C
C THIS ROUTINE ESTIMATES THE REDUCED SAT. LIQUID DENSITY AT A GIVEN    C
C REDUCED TEMPERATURE T/TC USING A POWER-LAW FORMULA THAT INTERPOLATES C
C FROM THE HIGHEST T WHERE PSAT IS ABLE TO SOLVE TO THE CRITICAL POINT C
C SHOULD ONLY BE CALLED FOR T .GT. 647.094866 K (TR.GT.(1.-1.7524E-6)) C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TR      R    I   REDUCED TEMPERATURE T/TC                            C
C DLSATC  R    O   REDUCED SATURATED LIQUID DENSITY RHO/RHOC           C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 23MAY96 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double A1 = 1.281714; 
    static const double BETA = 0.326;
    
    double TAU;
    double dlsatc;
    
    TAU = 1.0 - TR;
    dlsatc = 1.0 + A1*pow(TAU, BETA);
    
    return dlsatc;
}


double DVSATC(double TR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** DVSATC ***                                   C
C THIS ROUTINE ESTIMATES THE REDUCED SAT. VAPOR  DENSITY AT A GIVEN    C
C REDUCED TEMPERATURE T/TC USING A POWER-LAW FORMULA THAT INTERPOLATES C
C FROM THE HIGHEST T WHERE PSAT IS ABLE TO SOLVE TO THE CRITICAL POINT C
C SHOULD ONLY BE CALLED FOR T .GT. 647.094866 K (TR.GT.(1.-1.7524E-6)) C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TR      R    I   REDUCED TEMPERATURE T/TC                            C
C DVSATC  R    O   REDUCED SATURATED VAPOR  DENSITY RHO/RHOC           C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 23MAY96 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double A2 = 1.293771;
    static const double BETA = 0.326;
      
    double TAU;
    double dvsatc;
    
    TAU = 1.0 - TR;
    dvsatc = 1.0 - A2*pow(TAU, BETA);
      
    return dvsatc;
}


void PVAP(double TR, double *PS, double *DPSDT)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** PVAP ***                                     C
C THIS ROUTINE ESTIMATES THE REDUCED VAPOR PRESSURE P/PC AT A GIVEN    C
C REDUCED TEMPERATURE T/TC USING THE EQUATION OF SAUL AND WAGNER (J.   C
C PHYS. CHEM. REF. DATA, 16, 893 (1987) [AS MODIFIED TO 1990 T-SCALE   C
C BY WAGNER AND PRUSS, J. PHYS. CHEM. REF. DATA, 22, 783 (1993)])      C
C THE FIRST DERIVATIVE WITH RESPECT TO TR IS ALSO RETURNED.            C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TR      R    I   REDUCED TEMPERATURE T/TC                            C
C PS      R    O   REDUCED SATURATION PRESSURE, PSAT/PC                C
C DPSDT   R    O   FIRST DERIVATIVE OF PS WITH RESPECT TO TR           C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 17JUL95 - INITIAL CREATION BY AHH                                    C
C 11FEB97 - AHH: MAKE EXP'S UNIFORMLY DOUBLE PRECISION                 C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double A[] = { 0.0, -7.85951783, 1.84408259, -11.7866497, 22.6807411, 
                                -15.9618719, 1.80122502 };
	double TAU, SUM, PSLN, DSDTAU, DLNPDT;
     
    TAU = 1.0 - TR;
    SUM = TAU * (A[1]  + TAU*TAU*(A[3] + A[5]*TAU)
        + sqrt(TAU)*(A[2] + TAU*TAU*(A[4] + A[6]*pow(TAU, 4)))); 

    PSLN = SUM / TR;
    *PS = exp(PSLN);
    DSDTAU = A[1]  + TAU*TAU*(3.0*A[3] + 4.0*A[5]*TAU) + sqrt(TAU)*(1.50*A[2] 
           + 3.5*A[4]*TAU*TAU + 7.5*A[6]*pow(TAU, 6));
    DLNPDT = -(PSLN + DSDTAU) / TR;
    *DPSDT = *PS * DLNPDT;
      
    return;
}

double TVAP(double PR)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** TVAP ***                                     C
C THIS ROUTINE ESTIMATES THE REDUCED TEMPERATURE T/TC AT A GIVEN       C
C REDUCED VAPOR PRESSURE P/PC USING THE EQUATION OF SAUL AND WAGNER (J.C
C PHYS. CHEM. REF. DATA, 16, 893 (1987) [AS MODIFIED TO 1990 T-SCALE   C
C BY WAGNER AND PRUSS, J. PHYS. CHEM. REF. DATA, 22, 783 (1993)])      C
C IT ITERATIVELY CALLS PVAP, WHICH CONTAINS THIS EQUATION              C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C PR      R    I   REDUCED SATURATION PRESSURE, PSAT/PC                C
C TS      R    O   REDUCED TEMPERATURE T/TC                            C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 17JUL95 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double TOL = 1.0e-7;
    
    double PRLOG, PL, TG, PSNEW, DPSDT, PERRLN, TGINV, DLNPTI, TGINV2;
      
    PRLOG = log(PR);
/*
C FOR INITIAL GUESS, USE GUESS FROM HGK ROUTINE TSAT
*/
    PL = 7.1226152 + 2.302585*PRLOG;     
    TG = 0.57602 + PL*(0.042887 + PL*(0.00368 + PL*(3.837e-4 + PL*3.0e-5)));
            
    if (TG < 0.422) 
    	TG = 0.422;
    	
    if (TG > 0.9996) 
    	TG = 0.9996;
    	
STNO_11:
    PVAP(TG, &PSNEW, &DPSDT);		// get new iterate value for PSNEW (and hence TG).        

	if (fabs((PSNEW - PR)/PR) < TOL)  goto STNO_777;

    PERRLN = PRLOG - log(PSNEW);
    TGINV = 1.0 / TG;
    DLNPTI = -TG*TG*DPSDT/PSNEW;
    TGINV2 = TGINV + PERRLN/DLNPTI;
    TG = 1.0 / TGINV2;			
      	
    if (TG >= 1.0)  TG = 0.999999;
    	
    goto STNO_11;
    
STNO_777:
    return TG;
}


int PMELT1(double TK, double *PMPA)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** PMELT1 ***                                   C
C THIS ROUTINE GIVES THE MELTING PRESSURE PMPA AS A FUNCTION OF        C
C TEMPERATURE TK USING THE EQUATION OF WAGNER ET AL.  (J. PHYS.        C
C CHEM. REF. DATA, 23, 515 (1994).  FOR COEXISTENCE WITH ICE I         C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TK      R    I   TEMPERATURE IN K                                    C
C PMPA    R    O   MELTING PRESSURE IN MPA                             C
C IERR    I    O   RETURN CODE:                                        C
C                  0 = SUCCESS                                         C
C                  1 = TEMPERATURE BELOW VALID RANGE (251.165 K)       C
C                  2 = TEMPERATURE ABOVE VALID RANGE (273.16 K)        C
C                  IF 1 OR 2, PMPA IS RETURNED AS ZERO                 C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 28SEP95 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double TN = 273.16; 
    static const double TMIN = 251.165; 
    static const double PN = 611.657e-6;

    int IERR;
    double THETA, TERM2, TERM3, PRAT;
      
    *PMPA = 0.0;
    IERR = 0;
      
    if (TK < TMIN) 
    	IERR = 1;
    else if (TK > TN) 
        IERR = 2;
    else {
        THETA = TK / TN;
        TERM2 = -0.626e6 * (1.0 - pow(THETA, -3));
        TERM3 = 0.197135e6 * (1.0 - pow(THETA, 21.2));
        PRAT = 1.0 + TERM2 + TERM3;
        *PMPA = PRAT * PN;
    }
      
    return IERR;
}


int PMELT2(double TK, double *PMPA, int *IFORM)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** PMELT2 ***                                   C
C THIS ROUTINE GIVES THE MELTING PRESSURE PMPA AS A FUNCTION OF        C
C TEMPERATURE TK USING THE EQUATION OF WAGNER ET AL.  (J. PHYS.        C
C CHEM. REF. DATA, 23, 515 (1994).  FOR COEXISTENCE WITH HIGHER-P      C
C (NON-I) FORMS OF ICE                                                 C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C TK      R    I   TEMPERATURE IN K                                    C
C PMPA    R    O   MELTING PRESSURE IN MPA                             C
C IFORM   I    O   FORM OF ICE FOR WHICH EQUILIBRIUM EXISTS            C
C IERR    I    O   RETURN CODE:                                        C
C                  0 = SUCCESS                                         C
C                  1 = TEMPERATURE BELOW VALID RANGE (251.165 K)       C
C                  2 = TEMPERATURE ABOVE VALID RANGE (715. K)          C
C                  IF 1 OR 2, PMPA IS RETURNED AS ZERO                 C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 28SEP95 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
    static const double TMIN3 = 251.165; 
    static const double PN3 = 209.9;
    static const double TMIN5 = 256.164; 
    static const double PN5 = 350.1;
    static const double TMIN6 = 273.31;  
    static const double PN6 = 632.4;
    static const double TMIN7 = 355.0;    
    static const double PN7 = 2216.0; 
    static const double TMAX7 = 715.0;
      
    int IERR;
    double THETA, PRAT, TERM1, TERM2, TERM3;
      
    *PMPA = 0.0;
    IERR = 0;
    *IFORM = 0;
      
    if (TK < TMIN3) 
    	IERR = 1;
    else if (TK > TMAX7) 
        IERR = 2;
    else if (TK <= TMIN5) {
        THETA = TK / TMIN3;
        PRAT = 1.0 - 0.295252*(1.0 - pow(THETA, 60));
        *PMPA = PRAT * PN3;
        *IFORM = 3;
    } 
    else if (TK <= TMIN6) {
        THETA = TK / TMIN5;
        PRAT = 1.0 - 1.18721*(1.0 - pow(THETA, 8));
        *PMPA = PRAT * PN5;
        *IFORM = 5;
    }
    else if (TK <= TMIN7) {
        THETA = TK / TMIN6;
        PRAT = 1.0 - 1.07476*(1.0 - pow(THETA, 4.6));
        *PMPA = PRAT * PN6;
        *IFORM = 6;
    }
    else {
        THETA = TK / TMIN7;
        TERM1 = 1.73683 * (1.0 - 1.0/THETA);
        TERM2 = -0.0544606 * (1.0 - pow(THETA, 5));
        TERM3 = 0.806106e-7 * (1.0 - pow(THETA, 22));
        PRAT = exp(TERM1 + TERM2 + TERM3);
        *PMPA = PRAT * PN7;
        *IFORM = 7;
      }
      
    return IERR;
}


void IVZERO(int IVEC[], int ILEN)
{
/*
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                     *** IVZERO ***                                   C
C THIS ROUTINE ZEROS OUT AN INTEGER VECTOR OF LENGTH ILEN              C
C                                                                      C
C ARGUMENT LIST:                                                       C
C  NAME  TYPE I/O? EXPLANATION                                         C
C ------ ---- ---- --------------------------------------------------  C
C IVEC    IA  I/O  INTEGER VECTOR TO BE ZEROED                         C
C ILEN    I    I   LENGTH OF IVEC                                      C
C                                                                      C
C MAINTENANCE:                                                         C
C                                                                      C
C 12JUL95 - INITIAL CREATION BY AHH                                    C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
*/
	for (int i=0; i<=ILEN; i++) 	
   		IVEC[i] = 0;
  
    return;
}


