#include <math.h>
#include <stdlib.h> /* for integer absolute value function: abs(i)  */

#include "nist.h"
#include "intpk.h"
#include "solvpk.h"

/*
    Function declarations from file "auxpk.c" that are used in INTPK.
*/
int PMELT1(double TK, double *PMPA);
int PMELT2(double TK, double *PMPA, int *IFORM);

/*
    Function declarations for functions that are defined in PROPPK and used in INTPK.
*/
void PROPS1(int IWANT[], int IGFLG, double TK, double RHO, double PROPSI[], double PROPR[]);
void RIND(double TK, double RHO, double ALAM[], double RINDX[], int IERLAM[], int N);

/*
    Function declarations for functions that are defined and only used in INTPK.
*/
int TSUB(double *TK, double PMPA);

int TMELT(double *TK, double PMPA, int *IFORM);

int PMELT(double TK, int *NROOTS, double *PMPA1, double *PMPA2, int *IFORM);

void BNDCHK(double TK, double PMPA, double RHO, int MODE, int ISCHK, int *ISFLG, int ICCHK,
            int *ICFLG, int IPCHK[], int IPFLG[], int IWORK[], double PROPSI[], double RWORK[]);

int PSUB(double TK, double *PMPA);

/*
    Beginning of function definitions....
*/

int DENS(double *DOUT, double PMPA, double D, double TK, double *DPD, int IWORK[], double PROPR[])
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** DENS  ***                                    C
    C THIS ROUTINE FINDS THE DENSITY (IN KG/M3) CORRESPONDING TO A GIVEN   C
    C TEMPERATURE TK (IN K) AND PRESSURE PMPA (IN MPA).  AN                C
    C INITIAL GUESS (D) IS REQUIRED, AND THE VALUE OF DPDD IS ALSO         C
    C RETURNED.                                                            C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C DOUT    R    O   DENSITY AT TK AND PMPA, KG/M3                       C
    C PMPA    R    I   PRESSURE, MPA                                       C
    C D       R    I   INITIAL GUESS FOR DENSITY, KG/M3                    C
    C TK      R    I   TEMPERATURE, K                                      C
    C DPD     R    O   FIRST DERIVATIVE OF PMPA WITH RESPECT TO D AT DOUT  C
    C IWORK   IA   -   INTEGER VECTOR FOR USE BY LOWER-LEVEL ROUTINES      C
    C PROPR   RA   -   VECTOR OF REDUCED PROPERTIES USED BY LOWER-LEVEL    C
    C                  ROUTINES                                            C
    C IERR    I    O   DFIND RETURN STATUS FLAG.  NEGATIVES FROM DFIND1,   C
    C                  POSITIVES FROM DFIND2.  MEANINGS:                   C
    C                  0: CONVERGED                                        C
    C                 -2: UNABLE TO BOUND ROOT (DFIND1)                    C
    C                 -3: UNABLE TO CONVERGE BOUNDED ROOT (DFIND1)         C
    C                  1: NO ROOT FOR REQUESTED PHASE.  RETURNS D WHERE    C
    C                     DPD = 0 (DFIND2)                                 C
    C                  2: UNABLE TO BOUND ROOT (DFIND2)                    C
    C                  3: UNABLE TO CONVERGE BOUNDED ROOT (DFIND2)         C
    C                  5: INPUT PRESSURE NOT IN VALID RANGE                C
    C                  6: INPUT TEMPERATURE NOT IN VALID RANGE             C
    C                  7: INPUT T,P TOO FAR IN SUBCOOLED SOLID REGION.     C
    C                     CALCULATIONS ABORTED                             C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 07SEP95 - INITIAL CREATION BY AHH                                    C
    C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
    C 29SEP95 - AHH: CHANGE IWANT TO IWORK, REDOCUMENT ACCORDINGLY         C
    C 19OCT95 - AHH: CHECK FOR VIOLATION OF ABSOLUTE T AND P LIMITS        C
    C 15FEB96 - AHH: CHECK FOR TOO FAR INTO SUBCOOLED SOLID REGION         C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */

    int IERR, IERS, IFORM;
    double TSOLID, DR, PR, TR, DOUTR, DPDR;

    /*
    C CHECK T AND P AGAINST LIMITS OF VALIDITY
    C IF INVALID, RETURN THE INITIAL GUESS
    C
    */
    if ((PMPA < PLOWER) || (PMPA > PUPPER))
    {
        IERR = 5;
        *DOUT = D;
        *DPD = 0.0;
        return IERR;
    }

    if ((TK < TLOWER) || (TK > TUPPER))
    {
        IERR = 6;
        *DOUT = D;
        *DPD = 0.0;
        return IERR;
    }

    if (PMPA <= PTRIPW)
    {

        IERS = TSUB(&TSOLID, PMPA);

        if ((IERS != 1) && (TK < (TSOLID - 35.0)))
        {
            IERR = 7;
            *DOUT = 1.e-20;
            *DPD = 0.0;
            return IERR;
        }
    }
    else
    {

        IERS = TMELT(&TSOLID, PMPA, &IFORM);

        if ((IERS != 2) && (TK < (TSOLID - 35.0)))
        {
            IERR = 7;
            *DOUT = 1.0e-20;
            *DPD = 0.0;
            return IERR;
        }
    }

    DR = D / RHOCW;
    PR = PMPA / PCW;
    TR = TK / TCW;

    IERR = DFIND(&DOUTR, PR, DR, TR, &DPDR, IWORK, PROPR);

    *DOUT = DOUTR * RHOCW;
    *DPD = DPDR * PCW / RHOCW;

    return IERR;
}

int DENS0(double *DOUT, double PMPA, double TK, double *DPD, int IWORK[], double PROPR[])
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** DENS0 ***                                    C
    C THIS ROUTINE FINDS THE DENSITY (IN KG/M3) CORRESPONDING TO A GIVEN   C
    C TEMPERATURE (IN K) TK AND PRESSURE (IN MPA) PMPA.  IN                C
    C CONTRAST TO DENS, NO INITIAL GUESS IS REQUIRED (ONE IS GENERATED    C
    C INTERNALLY).  THE VALUE OF DPD IS ALSO RETURNED.                     C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C DOUT    R    O   DENSITY AT TK AND PMPA, KG/M3                       C
    C PMPA    R    I   PRESSURE, MPA                                       C
    C TK      R    I   TEMPERATURE, K                                      C
    C DPD     R    O   FIRST DERIVATIVE OF PMPA WITH RESPECT TO D AT DOUT  C
    C IWORK   IA   -   INTEGER VECTOR FOR USE BY LOWER-LEVEL ROUTINES      C
    C PROPR   RA   -   VECTOR OF REDUCED PROPERTIES USED BY LOWER-LEVEL    C
    C                  ROUTINES                                            C
    C IERR    I    O   DFIND0 RETURN STATUS FLAG.  NEGATIVES FROM DFIND1,  C
    C                  POSITIVES FROM DFIND2.  MEANINGS:                   C
    C                  0: CONVERGED                                        C
    C                 -2: UNABLE TO BOUND ROOT (DFIND1)                    C
    C                 -3: UNABLE TO CONVERGE BOUNDED ROOT (DFIND1)         C
    C                  1: NO ROOT FOR REQUESTED PHASE.  RETURNS D WHERE    C
    C                     DPD = 0 (DFIND2)                                 C
    C                  2: UNABLE TO BOUND ROOT (DFIND2)                    C
    C                  3: UNABLE TO CONVERGE BOUNDED ROOT (DFIND2)         C
    C                  5: INPUT PRESSURE NOT IN VALID RANGE                C
    C                  6: INPUT TEMPERATURE NOT IN VALID RANGE             C
    C                  7: INPUT T,P TOO FAR IN SUBCOOLED SOLID REGION.     C
    C                     CALCULATIONS ABORTED                             C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 07SEP95 - INITIAL CREATION BY AHH                                    C
    C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
    C 29SEP95 - AHH: CHANGE IWANT TO IWORK, REDOCUMENT ACCORDINGLY         C
    C 19OCT95 - AHH: CHECK FOR VIOLATION OF ABSOLUTE T AND P LIMITS        C
    C 15FEB96 - AHH: CHECK FOR TOO FAR INTO SUBCOOLED SOLID REGION         C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    int IERR, IERS, IFORM;
    double TSOLID, TR, PR, DOUTR, DPDR;

    /*
    C CHECK T AND P AGAINST LIMITS OF VALIDITY
    C IF INVALID, RETURN A SMALL NUMBER
    */
    if ((PMPA < PLOWER) || (PMPA > PUPPER))
    {
        IERR = 5;
        *DOUT = 1.0e-20;
        *DPD = 0.0;
        return IERR;
    }

    if ((TK < TLOWER) || (TK > TUPPER))
    {
        IERR = 6;
        *DOUT = 1.0e-20;
        *DPD = 0.0;
        return IERR;
    }

    if (PMPA <= PTRIPW)
    {
        IERS = TSUB(&TSOLID, PMPA);

        if ((IERS != 1) && (TK < (TSOLID - 35.0)))
        {
            IERR = 7;
            *DOUT = 1.0e-20;
            *DPD = 0.0;
            return IERR;
        }
        else
        {

            IERS = TMELT(&TSOLID, PMPA, &IFORM);

            if ((IERS != 2) && (TK < (TSOLID - 35.0)))
            {
                IERR = 7;
                *DOUT = 1.0e-20;
                *DPD = 0.0;
                return IERR;
            }
        }
    }

    TR = TK / TCW;
    PR = PMPA / PCW;

    IERR = DFIND0(&DOUTR, PR, TR, &DPDR, IWORK, PROPR);

    *DOUT = DOUTR * RHOCW;
    *DPD = DPDR * PCW / RHOCW;

    return IERR;
}

void PROPS(int IWANT[], double TK, double RHO, double PROPSI[], double PROPR[],
           int I2PHCK, int *I2PH, int ISCHK, int *ISFLG, int ICCHK, int *ICFLG,
           int IPCHK[], int IPFLG[], int IGFLG, int NRI, double WAVRI[],
           double RI[], int IRIFLG[])
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** PROPS ***                                    C
    C THIS ROUTINE COMPUTES ALL PROPERTIES REQUESTED VIA THE IWANT VECTOR  C
    C AT A GIVEN TEMPERATURE AND DENSITY.  PROPERTIES ARE RETURNED IN SI   C
    C UNITS: KG, M3, K, MPA, KJ.  THE LOWER-LEVEL ROUTINE PROPS1 IS        C
    C CALLED TO RETURN THE PROPERTIES.  IT ALSO CHECKS TO SEE IF THE POINT C
    C IS IN THE 2-PHASE REGION AND RETURNS THE APPROPRIATE QUALITY AND     C
    C BULK PROPERTIES.  BOUNDS CHECKING IS ALSO DONE HERE IF REQUESTED     C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C IWANT   IA   I   FOR WHETHER OR NOT TO COMPUTE VARIOUS PROPERTIES,   C
    C                  SEE PROPSI ARRAY BELOW FOR NUMBERING OF PROPERTIES. C
    C                  1=COMPUTE, 0=DON'T COMPUTE                          C
    C TK      R    I   TEMPERATURE IN KELVINS                              C
    C RHO     R    I   DENSITY IN KG/M3                                    C
    C PROPSI  RA   O   VECTOR OF PROPERTIES IN SI UNITS AS REQUESTED BY    C
    C                  IWANT.  NUMBERING OF PROPERTIES:                    C
    C                  1: TEMPERATURE IN K                                 C
    C                  2: PRESSURE IN MPA                                  C
    C                  3: DENSITY IN KG/M3                                 C
    C                  4: VOLUME IN M3/KG                                  C
    C                  5: QUALITY (ONLY SET IF 2-PHASE)                    C
    C                  6: ENTHALPY IN KJ/KG                                C
    C                  7: ENTROPY IN KJ/(KG*K)                             C
    C                  8: ISOCHORIC HEAT CAPACITY IN KJ/(KG*K)             C
    C                  9: ISOBARIC HEAT CAPACITY IN KJ/(KG*K)              C
    C                 10: INTERNAL ENERGY IN KJ/KG                         C
    C                 11: HELMHOLTZ ENERGY IN KJ/KG                        C
    C                 12: GIBBS ENERGY IN KJ/KG                            C
    C                 13: FUGACITY IN MPA                                  C
    C                 14: ISOTHERMAL COMPRESSIBILITY IN MPA**(-1)          C
    C                 15: VOLUME EXPANSIVITY IN K**(-1)                    C
    C                 16: DP/DRHO (CONSTANT T) IN MPA/(KG/M3)              C
    C                 17: DP/DT (CONSTANT RHO) IN MPA/K                    C
    C                 18: SPEED OF SOUND IN M/SEC                          C
    C                 19: JOULE-THOMSON COEFFICIENT IN K/MPA               C
    C                 20: THERMAL CONDUCTIVITY IN W/M-K                    C
    C                 21: VISCOSITY IN PA-SEC                              C
    C                 22: STATIC DIELECTRIC CONSTANT (RETURNED AS -999. IF C
    C                     CALCULATION COULD NOT BE DONE)                   C
    C                 23: 2ND DERIVATIVE D2P/DRHO2 (CONSTANT T) IN         C
    C                     MPA/(KG/M3)**2                                   C
    C                 24: 2ND DERIVATIVE D2P/DT2 (CONSTANT RHO)            C
    C                     IN MPA/K**2                                      C
    C                 25: 2ND DERIVATIVE D2P/DRHODT (CONSTANT T, RHO)      C
    C                     IN MPA/(KG/M3)/K                                 C
    C                 26: DRHO/DT (CONSTANT P) IN (KG/M3)/K                C
    C                 27: DRHO/DP (CONSTANT T) IN (KG/M3)/MPA              C
    C                 28: 2ND DERIVATIVE D2RHO/DT2 (CONSTANT P)            C
    C                     IN (KG/M3)/K**2                                  C
    C                 29: 2ND DERIVATIVE D2RHO/DP2 (CONSTANT T)            C
    C                     IN (KG/M3)/MPA**2                                C
    C                 30: 2ND DERIVATIVE D2RHO/DTDP (CONSTANT P, T)        C
    C                     IN (KG/M3)/K/MPA                                 C
    C                 31: DIELECTRIC DERIVATIVE D(EPS)/D(RHO) (CONSTANT T) C
    C                     IN M3/KG                                         C
    C                 32: DIELECTRIC DERIVATIVE D(EPS)/DT (CONSTANT RHO)   C
    C                     IN K**(-1)                                       C
    C                 33: DIELECTRIC DERIVATIVE D(EPS)/DT (CONSTANT P)     C
    C                     IN K**(-1)                                       C
    C                 34: DIELECTRIC DERIVATIVE D(EPS)/DP (CONSTANT T)     C
    C                     IN MPA**(-1)                                     C
    C                 35: 2ND DIELECTRIC DERIVATIVE D2(EPS)/DT2 (CONST. P) C
    C                     IN K**(-2)                                       C
    C                 36: 2ND DIELECTRIC DERIVATIVE D2(EPS)/DP2 (CONST. T) C
    C                     IN MPA**(-2)                                     C
    C                 37: 2ND DIELECTRIC DERIVATIVE D2(EPS)/DTDP           C
    C                     (CONSTANT P, T) IN (K*MPA)**(-1)                 C
    C                 38: DEBYE-HUCKEL SLOPE FOR ACTIVITY COEFFICIENT      C
    C                     IN (KG/MOL)**0.5                                 C
    C                 39: D-H SLOPE FOR OSMOTIC COEFF. IN (KG/MOL)**0.5    C
    C                 40: D-H SLOPE FOR APPARENT MOLAR VOLUME              C
    C                     IN (M3/MOL)*(KG/MOL)**0.5                        C
    C                 41: D-H SLOPE FOR APPARENT MOLAR ENTHALPY, DIVIDED   C
    C                     BY RT, IN (KG/MOL)**0.5                          C
    C                 42: D-H SLOPE FOR APPARENT MOLAR COMPRESSIBILITY     C
    C                     IN (M3/MOL/MPA)*(KG/MOL)**0.5                    C
    C                 43: D-H SLOPE FOR APPARENT MOLAR HEAT CAPACITY,      C
    C                     DIVIDED BY R, IN (KG/MOL)**0.5                   C
    C PROPR   RA   -  WORK VECTOR FOR REDUCED PROPERTIES PASSED TO PROPS1  C
    C I2PHCK  I    I  FLAG FOR WHETHER TO CHECK AND SEE IF REQUESTED T AND C
    C                 RHO IN 2-PHASE REGION.  0=DON'T CHECK, 1=CHECK       C
    C I2PH    I    O  OUTPUT FLAG FOR RELATION TO 2-PHASE REGION           C
    C                 0 = I2PHCK=0 (OR IDEAL GAS)                          C
    C                -1 = SUBCOOLED LIQUID (ALSO T<TC AND P>PC)            C
    C                 1 = SUPERHEATED VAPOR (ALSO T>TC)                    C
    C                 2 = 2-PHASE REGION.  QUALITY IS PUT IN SLOT 5, PROPS C
    C                     RETURNED IN PROPSI ARE APPROPRIATE AVERAGES. FOR C
    C                     PROPS THAT CAN'T BE AVERAGED, ZERO IS RETURNED   C
    C                 3 = UNABLE TO PERFORM 2-PHASE CHECK BECAUSE TK BELOW C
    C                     MINIMUM.  PROPS RETURNED AS THOUGH 1-PHASE       C
    C                 4 = SAME AS 2, EXCEPT 2-PHASE ENVELOPE EXTRAPOLATED  C
    C                     TO REGION BELOW TRIPLE POINT                     C
    C *** NOTE:  FOLLOWING ARE FLAGS FOR WHETHER TO MAKE A CHECK (1=CHECK, C
    C            0=DON'T) AND FLAGS RETURNED FROM THE CHECKS (0=OK OR NOT  C
    C            CHECKED, 1=EXTRAPOLATED BEYOND RECOMMENDED REGION, 2=     C
    C            UNACCEPTABLY OUTSIDE OF REGION                            C
    C ISCHK   I    I   WHETHER TO CHECK IF EQUILIBRIUM PHASE IS SOLID      C
    C ISFLG   I    O   RESULT OF SOLID CHECK                               C
    C ICCHK   I    I   WHETHER TO CHECK IF TOO NEAR CRITICAL POINT         C
    C ICFLG   I    O   RESULT OF NEAR-CRITICAL CHECK                       C
    C IPCHK   IA   I   WHETHER TO CHECK BOUNDARIES FOR VARIOUS PROPERTIES. C
    C                  ARRAY ELEMENTS ARE: 1: THERMODYNAMIC PROPERTIES     C
    C                                      2: VISCOSITY                    C
    C                                      3: THERMAL CONDUCTIVITY         C
    C                                      4: DIELECTRIC CONSTANT          C
    C                                      5: REFRACTIVE INDEX             C
    C IPFLG   IA   O   RESULTS OF PROPERTY BOUNDARY CHECKS                 C
    C IGFLG   I    I   WHETHER TO COMPUTE AND RETURN ONLY IDEAL-GAS PROPS  C
    C                  1=IDEAL GAS, 0=REAL FLUID                           C
    C NRI     I    I   NUMBER OF REFRACTIVE INDEX POINTS TO COMPUTE        C
    C WAVRI   RA   I   ARRAY OF WAVELENGTHS (IN MICROMETERS) AT WHICH TO   C
    C                  COMPUTE REFRACTIVE INDEX                            C
    C RI      RA   O   RETURNED ARRAY OF REFRACTIVE INDICES                C
    C IRIFLG  IA   O   ARRAY OF RETURN FLAGS FOR REF. INDEX WAVELENGTHS    C
    C                  0     = OK                                          C
    C                  1(-1) = LARGER(SMALLER) THAN RECOMMENDED RANGE,     C
    C                          VALUE IS AN EXTRAPOLATION                   C
    C                  2(-2) = TOO LARGE(SMALL), BEYOND REASONABLE         C
    C                          EXTRAPOLATION RANGE. WILL STILL RETURN      C
    C                          COMPUTED NUMBER IF WITHIN RANGE WHERE CALC. C
    C                          DOESN'T BLOW UP, OTHERWISE RETURNS ZERO.    C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 28SEP95 - INITIAL CREATION BY AHH                                    C
    C 29SEP95 - UPGRADE HANDLING OF QUALITY                                C
    C 05OCT95 - AHH: PUT IN BOUNDS CHECKING TYPES 2 AND 3                  C
    C 27OCT95 - AHH: FIX BOUNDS CHECKING AND SETTING OF I2PH FLAG          C
    C 09NOV95 - AHH: REDO BOUNDS CHECKING WITH CALL TO BNDCHK              C
    C 27NOV95 - AHH: ADD DIELECTRIC CONSTANT IN SLOT 21                    C
    C 07DEC95 - AHH: ADD VOLUME EXPANSIVITY AS PROPERTY 22                 C
    C 07DEC95 - AHH: IMPLEMENT CALCULATION OF CV IN 2-PHASE REGION         C
    C 11DEC95 - AHH: REORDER PROPERTY NUMBERING                            C
    C 13DEC95 - AHH: FIX 2-PHASE CV CALCULATION                            C
    C 07JUN96 - AHH: IF T<TC AND P>PC, CALL IT SUBCOOLED                   C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C 17SEP96 - AHH: EXPUNGE TAB CHARACTERS                                C
    C 07NOV96 - AHH: CORRECT SOME COMMENTS, PARTICULARLY ABOUT QUALITY     C
    C 09JAN97 - AHH: ADD IGFLG TO ARGUMENT LIST TO COMPUTE I.G. PROPS.     C
    C 13JAN97 - AHH: ALLOW V/L CALCS BELOW TRIPLE POINT                    C
    C 26FEB97 - AHH: ADD DERIVATIVE PROPERTIES IN SLOTS 23-30              C
    C 05MAR97 - AHH: ADD DIELECTRIC DERIVATIVES IN SLOTS 31-37             C
    C 06MAR97 - AHH: ADD DEBYE-HUCKEL SLOPES IN SLOTS 38-43                C
    C 25MAR97 - AHH: ADD REFRACTIVE INDEX CALCULATIONS                     C
    C 20MAY97 - AHH: ADD I2PH = 4 CAPABILITY                               C
    C 30MAY97 - AHH: CHANGED NAME TO PROPS TO PRESERVE OLD PROPS0 ARG LIST C
    C 01JUL97 - AHH: DO BOUNDARY CHECKS IF 2-PHASE BELOW TRIPLE POINT      C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C
    C NEED CONSTANTS FOR WATER (now located in "nist.h")
    C
    C
    C NEED THREE TEMPORARY ARRAYS FOR VARIOUS PURPOSES
    */
    double TPROP[NPROP + 1];
    int IWTMP[NPROP + 1];
    int IPDUM[6];

    int ICLIQ, ICVAP, IERRPS, IPMAX;
    double Q, RHOV, RHOL, Q1, VVAP, VLIQ, DPSDT, DVVDT, DVLDT, DUVDP, DULDP, DUVDT, DULDT;
    double DUVDTS, DULDTS, DQDT, DELU, CV2PH, PDUM, PMPA;

    /*
    C CHECK FOR 2-PHASE (V-L) REGION IF REQUESTED
    C DON'T DO THIS CHECK IF IDEAL GAS PROPERTIES REQUESTED
    */
    *I2PH = 0;

    if ((I2PHCK != 0) && (IGFLG != 1))
    {
        if (TK < TCW)
        {
            IERRPS = PSAT(TK, &PMPA, &RHOL, &RHOV, IWTMP, PROPR);
            /*
            C IF T TOO LOW TO CHECK V/L SATURATION CURVE, CAN'T GET COEXISTING
            C DENSITIES FROM PSAT TO CHECK.  SO SET RETURN CODE AND CALC AS
            C IF SINGLE PHASE
            */
            if (IERRPS == 1)
                *I2PH = 3;

            else if ((RHO < RHOL) && (RHO > RHOV))
            {
                /*
                C 2-PHASE, FIGURE QUALITY AND GET PROPERTIES AT PHASE BOUNDARIES
                */
                if (IERRPS == 4)
                    *I2PH = 4;
                else
                    *I2PH = 2;

                Q = (1.0 / RHO - 1.0 / RHOL) / (1.0 / RHOV - 1.0 / RHOL);

                for (int i = 1; i <= NPROP; i++)
                    IWTMP[i] = IWANT[i];

                /*
                C IF YOU ARE GOING TO BE FIGURING 2-PHASE CV, MAKE SURE YOU CALL FOR
                C ALL THE NEEDED PROPERTIES IN BOTH PHASES
                */
                if (IWANT[8] != 0)
                {
                    IWTMP[2] = 1;
                    IWTMP[6] = 1;
                    IWTMP[9] = 1;
                    IWTMP[10] = 1;
                    IWTMP[14] = 1;
                    IWTMP[15] = 1;
                }

                PROPS1(IWTMP, IGFLG, TK, RHOV, TPROP, PROPR);
                PROPS1(IWTMP, IGFLG, TK, RHOL, PROPSI, PROPR);

                /*
                C ZERO OUT QUANTITIES THAT ARE MEANINGLESS IN 2-PHASE REGION
                C FIGURE OTHERS IF REQUESTED - THOSE THAT ARE IDENTICAL IN BOTH PHASES
                C SHOULD ALREADY BE IN THE PROPSI ARRAY
                */
                Q1 = 1.0 - Q;

                if (IWANT[8] != 0)
                {
                    /*
                    C LENGTHY CALCULATION OF 2-PHASE CV.  FIRST, CALCULATE DPSAT/DT
                    C (HAVE TO TOSS IN A 1.D3 SINCE P IN MPA BUT H IN KJ)
                    */
                    VVAP = 1.0 / RHOV;
                    VLIQ = 1.0 / RHOL;
                    DPSDT = (TPROP[6] - PROPSI[6]) / (TK * (VVAP - VLIQ));
                    DPSDT = DPSDT / 1.0e3;
                    /*
                    C NEXT CALCULATE THE VARIATION OF THE SATURATED VOLUMES WITH T
                    */
                    DVVDT = VVAP * (TPROP[15] - TPROP[14] * DPSDT);
                    DVLDT = VLIQ * (PROPSI[15] - PROPSI[14] * DPSDT);
                    /*
                    C WE ALSO NEED THE VARIATIONS OF SATURATED INT. ENG. WITH T
                    */
                    DUVDP = 1.0e3 * VVAP * (TPROP[14] * TPROP[2] - TPROP[15] * TK);
                    DULDP = 1.0e3 * VLIQ * (PROPSI[14] * PROPSI[2] - PROPSI[15] * TK);
                    DUVDT = TPROP[9] - 1.0e3 * TPROP[2] * VVAP * TPROP[15];
                    DULDT = PROPSI[9] - 1.0e3 * PROPSI[2] * VLIQ * PROPSI[15];
                    DUVDTS = DUVDT + DUVDP * DPSDT;
                    DULDTS = DULDT + DULDP * DPSDT;
                    /*
                    C NOW YOU CAN CALCULATE ISOCHORIC DQ/DT, WHICH THEN GOES INTO CV
                    */
                    DQDT = -(Q * DVVDT + Q1 * DVLDT) / (VVAP - VLIQ);
                    DELU = TPROP[10] - PROPSI[10];
                    CV2PH = Q * DUVDTS + Q1 * DULDTS + DQDT * DELU;
                    PROPSI[8] = CV2PH;
                }

                if (IWANT[3] != 0)
                    PROPSI[3] = RHO;

                if (IWANT[4] != 0)
                    PROPSI[4] = 1.0 / RHO;

                if (IWANT[6] != 0)
                    PROPSI[6] = Q * TPROP[6] + Q1 * PROPSI[6];

                if (IWANT[7] != 0)
                    PROPSI[7] = Q * TPROP[7] + Q1 * PROPSI[7];

                if (IWANT[10] != 0)
                    PROPSI[10] = Q * TPROP[10] + Q1 * PROPSI[10];

                if (IWANT[11] != 0)
                    PROPSI[11] = Q * TPROP[11] + Q1 * PROPSI[11];

                PROPSI[9] = 0.0;

                for (int i = 14; i <= 43; i++)
                    PROPSI[i] = 0.0;

                if (NRI > 0)
                {
                    for (int i = 1; i <= NRI; i++)
                        RI[i] = 0.0;
                }
                /*
                C BOUNDS CHECKING - NEED TO DO IF IN 2-PHASE REGION BELOW TRIPLE PT.
                C ELSE, DO BOUNDS CHECKING ONLY FOR NEAR-CRITICAL (SINCE OTHER
                C BOUNDS CAN'T BE VIOLATED IN 2-PHASE REGION)
                */
                if (*I2PH == 4)
                {
                    /* NOTE: both ISFLG and ICFLG are pointers here as they were passed into PROPS as
                             pointers. They can be changed in BNDCHK.
                    */
                    BNDCHK(TK, PDUM, RHOV, 2, 0, ISFLG, 0, ICFLG, IPCHK, IPFLG, IWTMP,
                           TPROP, PROPR);
                    BNDCHK(TK, PDUM, RHOL, 2, 0, ISFLG, 0, ICFLG, IPCHK, IPDUM, IWTMP,
                           TPROP, PROPR);

                    for (int i = 1; i <= 5; i++)
                        IPFLG[i] = (IPFLG[i] > IPDUM[i]) ? IPFLG[i] : IPDUM[i];
                }
                else if (ICCHK == 1)
                {
                    for (int i = 1; i <= 5; i++)
                        IPDUM[i] = 0;

                    BNDCHK(TK, PDUM, RHOV, 2, 0, ISFLG, ICCHK, &ICVAP, IPDUM, IPFLG,
                           IWTMP, TPROP, PROPR);

                    BNDCHK(TK, PDUM, RHOL, 2, 0, ISFLG, ICCHK, &ICLIQ, IPDUM, IPFLG,
                           IWTMP, TPROP, PROPR);

                    *ICFLG = (ICVAP > ICLIQ) ? ICVAP : ICLIQ;
                }
                /*
                C IF ASKED FOR SOLIDS CHECKING, ALSO SET THAT FLAG IF IN 2-PHASE
                C REGION BELOW TRIPLE-POINT TEMPERATURE
                */
                if ((ISCHK == 1) && (*I2PH == 4))
                    *ISFLG = 1;
            }
        }
        else
        {
            /* ABOVE TC, SO CALL IT SUPERHEATED */
            *I2PH = 1;
        }
    }

    /*
    C IF YOU DIDN'T DO THE 2-PHASE CALCULATION, CALL PROPS1 DIRECTLY
    C BUT FIRST CHECK BOUNDS, AND DON'T CALL IF OUT OF BOUNDS
    */
    if ((*I2PH != 2) && (*I2PH != 4))
    {
        Q = 0.0;

        BNDCHK(TK, PDUM, RHO, 2, ISCHK, ISFLG, ICCHK, ICFLG, IPCHK, IPFLG, IWTMP,
               TPROP, PROPR);

        IPMAX = 0;

        for (int i = 1; i <= 5; i++)
            IPMAX = (IPMAX > IPFLG[i]) ? IPMAX : IPFLG[i];

        IPMAX = (IPMAX > *ISFLG) ? IPMAX : *ISFLG;

        if (IPMAX < 2)
        {
            PROPS1(IWANT, IGFLG, TK, RHO, PROPSI, PROPR);

            if (NRI > 0)
                RIND(TK, RHO, WAVRI, RI, IRIFLG, NRI);
        }
    }

    /*
    C IF REQUESTED, CALCULATED WHETHER SUPERHEATED, SUBCOOLED, ETC.
    C IF T>TC, WAS ALREADY SET I2PH=1 AT BOTTOM OF 2-PHASE CHECKING
    */
    if ((I2PHCK != 0) && (*I2PH == 0))
    {
        if ((PROPR[2] < 1.0) && (RHO <= RHOV))
            *I2PH = 1;
        else
            *I2PH = -1;
    }

    PROPSI[5] = Q;

    return;
}

void BNDCHK(double TK, double PMPA, double RHO, int MODE, int ISCHK, int *ISFLG, int ICCHK,
            int *ICFLG, int IPCHK[], int IPFLG[], int IWORK[], double PROPSI[], double RWORK[])
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** BNDCHK ***                                   C
    C THIS ROUTINE CHECKS TO SEE IF THE REQUESTED POINT IS AN EXTRAPOLATIONC
    C OR OUT OF RANGE FOR A VARIETY OF CRITERIA                            C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    I   TEMPERATURE IN KELVINS                              C
    C PMPA    R    I   PRESSURE IN MPA                                     C
    C RHO     R    I   DENSITY IN KG/M3                                    C
    C MODE    I    I   TELLS WHAT INFORMATION IS AVAILABLE ON ENTRY        C
    C                  0: T, P, RHO ALL KNOWN                              C
    C                  1: T AND P KNOWN, RHO UNKNOWN                       C
    C                  2: T AND RHO KNOWN, P UNKNOWN                       C
    C *** NOTE:  FOLLOWING ARE FLAGS FOR WHETHER TO MAKE A CHECK (1=CHECK, C
    C            0=DON'T) AND FLAGS RETURNED FROM THE CHECKS (0=OK OR NOT  C
    C            CHECKED, 1=EXTRAPOLATED BEYOND RECOMMENDED REGION, 2=     C
    C            UNACCEPTABLY OUTSIDE OF REGION                            C
    C ISCHK   I    I   WHETHER TO CHECK IF EQUILIBRIUM PHASE IS SOLID      C
    C ISFLG   I    O   RESULT OF SOLID CHECK                               C
    C ICCHK   I    I   WHETHER TO CHECK IF TOO NEAR CRITICAL POINT         C
    C ICFLG   I    O   RESULT OF NEAR-CRITICAL CHECK                       C
    C IPCHK   IA   I   WHETHER TO CHECK BOUNDARIES FOR VARIOUS PROPERTIES. C
    C                  ARRAY ELEMENTS ARE: 1: THERMODYNAMIC PROPERTIES     C
    C                                      2: VISCOSITY                    C
    C                                      3: THERMAL CONDUCTIVITY         C
    C                                      4: DIELECTRIC CONSTANT          C
    C                                      5: REFRACTIVE INDEX             C
    C IPFLG   IA   O   RESULTS OF PROPERTY BOUNDARY CHECKS                 C
    C IWORK   IA   -   INTEGER WORK ARRAY, USED IF P GETS CALCULATED       C
    C PROPSI  RA   O   VECTOR OF PROPERTIES IN SI UNITS AS REQUESTED BY    C
    C                  IWANT.  NUMBERING OF PROPERTIES:                    C
    C                  2: PRESSURE IN MPA                                  C
    C RWORK   RA   -   REAL WORK ARRAY, USED IF P GETS CALCULATED          C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 08NOV95 - INITIAL CREATION BY AHH                                    C
    C 21MAY96 - AHH: PUT IN NEAR-CRITICAL CHECK, MODIFY OTHER BOUNDARIES   C
    C           IN ACCORDANCE WITH NEWLY RECEIVED RELEASE LANGUAGE         C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C 08JAN97 - AHH: CHANGE IN ARGUMENT LIST OF PROPS1                     C
    C 25MAR97 - AHH: ADD CHECK FOR REFRACTIVE INDEX BOUNDARY               C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C
    C
    C NEED CONSTANTS FOR WATER
    */
    static const double RCMNS = 320.39;
    static const double RCPLS = 323.61;
    static const double TCMNS = 647.076;
    static const double TCPLS = 652.096;

    int IHAVEP, IHAVER, INEEDP, INEEDR, IERRD, IERR, IT, IP, IFORM;
    double TC, DPD, TSOLID;

    *ISFLG = 0;
    *ICFLG = 0;

    for (int i = 1; i <= 5; i++)
        IPFLG[i] = 0;

    TC = TK - 273.15;
    /*
    C FIRST USE MODE TO DETERMINE WHAT YOU HAVE AND WHAT YOU NEED
    */
    if (MODE == 0)
    {
        IHAVEP = 1;
        IHAVER = 1;
    }
    else if (MODE == 1)
    {
        IHAVEP = 1;
        IHAVER = 0;
    }
    else
    {
        IHAVEP = 0;
        IHAVER = 1;
    }

    if ((IHAVEP == 0) && ((ISCHK == 1) || (IPCHK[1] == 1) || (IPCHK[2] == 1) ||
                          (IPCHK[3] == 1) || (IPCHK[4] == 1) || (IPCHK[5] == 1)))
        INEEDP = 1;
    else
        INEEDP = 0;

    if ((IHAVER == 0) && ((ICCHK == 1) || (IPCHK[5] == 1)))
        INEEDR = 1;
    else
        INEEDR = 0;

    /*
    C CHECK FOR "BAD" VALUES OF T AND RHO
    */
    if (TK < 0.0)
    {
        if (ISCHK == 1)
            *ISFLG = 2;

        for (int i = 1; i <= 5; i++)
        {
            if (IPCHK[i] == 1)
                IPFLG[i] = 2;
        }
        return;
    }

    if (IHAVER == 1)
    {
        if (RHO <= 0.0)
        {
            for (int i = 1; i <= 5; i++)
            {
                if (IPCHK[i] == 1)
                    IPFLG[i] = 2;
            }
            return;
        }
    }
    /*
    C COMPUTE P OR RHO IF NEEDED
    */

    if (INEEDP == 1)
    {
        IVZERO(IWORK, NPROP);
        IWORK[2] = 1;
        PROPS1(IWORK, 0, TK, RHO, PROPSI, RWORK);
        /*
        - JMK:  Note that PMPA is changed here but that the chage will NOT go back to the calling program.
                   BUG????
        */
        PMPA = PROPSI[2];
    }

    if (INEEDR == 1)
        IERRD = DENS0(&RHO, PMPA, TK, &DPD, IWORK, RWORK);

    /*
    C CHECK FOR SOLID BOUNDARIES
    */
    if (ISCHK == 1)
    {
        /*
        C CHECK FOR WHETHER EQUILIBRIUM STATE WOULD BE A SOLID
        C DEPENDING ON PRESSURE, CHECK VERSUS SUBLIMATION OR FREEZING
        C STILL NEED TO DECIDE HOW MUCH SUBCOOLING IS "RELIABLE"
        */
        if (PMPA <= PTRIPW)
        {
            IERR = TSUB(&TSOLID, PMPA);

            if (IERR != 1)
            {
                if (TK < (TSOLID - 35.0))
                    *ISFLG = 2;
                else if (TK < TSOLID)
                    *ISFLG = 1;
            }
            else
            {
                IERR = TMELT(&TSOLID, PMPA, &IFORM);

                if (IERR != 2)
                {
                    if (TK < (TSOLID - 35.0))
                        *ISFLG = 2;
                    else if (TK < TSOLID)
                        *ISFLG = 1;
                }
            }
        }
    }

    /*
    C CHECK FOR NEAR-CRITICAL REGION
    */
    if ((ICCHK == 1) && ((IERRD == 0) || (abs(IERRD) == 3)))
    {
        if ((TK > TCMNS) && (TK < TCPLS) && (RHO > RCMNS) && (RHO < RCPLS))
            *ICFLG = 1;
    }

    /*
    C CHECKS FOR INDIVIDUAL PROPERTIES OR SETS OF PROPERTIES
    C
    C CHECK FOR BOUNDS OF WAGNER/PRUSS EQUATION
    */
    if (IPCHK[1] == 1)
    {
        IT = 0;
        IP = 0;

        if ((TK > TUPPER) || (TK < TLOWER))
            IT = 2;
        else if ((TK > 1273.15) || (TK < 273.15))
            IT = 1;

        if ((PMPA > PUPPER) || (PMPA < PLOWER))
            IP = 2;
        else if ((PMPA > 1.0e3) || (PMPA <= 0.0))
            IP = 1;

        IPFLG[1] = (IP > IT) ? IP : IT;
    }

    /*
    C CHECK FOR BOUNDS OF VISCOSITY CORRELATION
    */
    if (IPCHK[2] == 1)
    {
        IT = 0;
        IP = 0;

        if ((TK > TUPPER) || (TK < TLOWER))
            IT = 2;
        else if ((TC > 900.0) || ((TC > 600.0) && (PMPA > 300.0)) ||
                 ((TC > 150.0) && (PMPA > 350.0)) || (PMPA > 500.0) || (TC < 0.0))
            IT = 1;

        if ((PMPA > PUPPER) || (PMPA < PLOWER))
            IP = 2;
        else if (PMPA <= 0.0)
            IP = 1;

        IPFLG[2] = (IP > IT) ? IP : IT;
    }

    /*
    C CHECK FOR BOUNDS OF THERMAL CONDUCTIVITY CORRELATION
    */
    if (IPCHK[3] == 1)
    {
        IT = 0;
        IP = 0;

        if ((TK > TUPPER) || (TK < TLOWER))
            IT = 2;
        else if ((TC > 800.0) || ((TC > 400.0) && (PMPA > 100.0)) ||
                 ((TC > 250.0) && (PMPA > 150.0)) || ((TC > 125.0) && (PMPA > 200.0)) ||
                 (PMPA > 400.0) || (TC < 0.0))
            IT = 1;

        if ((PMPA > PUPPER) || (PMPA < PLOWER))
            IP = 2;
        else if (PMPA <= 0.0)
            IP = 1;

        IPFLG[3] = (IP > IT) ? IP : IT;
    }

    /*
    C CHECK FOR BOUNDS OF DIELECTRIC CONSTANT CORRELATION
    */
    if (IPCHK[4] == 1)
    {
        IT = 0;
        IP = 0;

        if ((TK > TUPPER) || (TK < TLOWER))
            IT = 2;
        else if ((TK > 873.0) || (TK < 238.0))
            IT = 1;

        if ((PMPA > PUPPER) || (PMPA < PLOWER))
            IP = 2;
        else if ((PMPA > 1.0e3) || (PMPA <= 0.0))
            IP = 1;

        IPFLG[4] = (IP > IT) ? IP : IT;
    }

    /*
    C CHECK FOR BOUNDS OF REFRACTIVE INDEX CORRELATION
    */
    if (IPCHK[5] == 1)
    {
        IT = 0;
        IP = 0;

        if ((TK > TUPPER) || (TK < TLOWER))
            IT = 2;
        else if ((TK > 773.15) || (TK < 261.15))
            IT = 1;

        if ((PMPA > PUPPER) || (PMPA < PLOWER))
            IP = 2;
        else if (RHO > 1060.0)
            IP = 1;

        IPFLG[5] = (IP > IT) ? IP : IT;
    }

    return;
}

int PSAT(double TK, double *PMPA, double *RHOL, double *RHOV, int IWORK[], double PROPR[])
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** PSAT ***                                     C
    C THIS ROUTINE COMPUTES THE COEXISTENCE PRESSURE (IN MPA) AT A         C
    C GIVEN TEMPERATURE (IN K).  IT ALSO RETURNS THE COEXISTING            C
    C DENSITIES.  IT CALLS PCOEX, WHICH WORKS IN REDUCED UNITS.            C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    I   TEMPERATURE, K                                      C
    C PMPA    R    O   COEXISTENCE PRESSURE, MPA                           C
    C RHOL    R    O   SATURATED LIQUID DENSITY, KG/M3                     C
    C RHOV    R    O   SATURATED VAPOR DENSITY, KG/M3                      C
    C IWORK   IA   -   INTEGER VECTOR FOR USE BY LOWER-LEVEL ROUTINES      C
    C PROPR   RA   -   VECTOR OF REDUCED PROPERTIES USED BY LOWER-LEVEL    C
    C                  ROUTINES                                            C
    C IERR    I    O   ERROR FLAG (FROM PCOEX)                             C
    C                  0 = SUCCESS                                         C
    C                  1 = TEMPERATURE OUT OF RANGE                        C
    C                  2 = UNABLE TO BRACKET (PROBABLY BECAUSE VERY CLOSE  C
    C                      TO TC) RETURN VALUES FROM SATURATION EQUATIONS  C
    C                  3 = UNABLE TO CONVERGE BOUNDED SOLUTION.  RETURN    C
    C                      LAST GUESS FOR SOLUTION                         C
    C                  4 = TEMPERATURE BELOW TRIPLE POINT BUT IN RANGE     C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 07SEP95 - INITIAL CREATION BY AHH                                    C
    C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
    C 29SEP95 - AHH: CHANGE IWANT TO IWORK, REDOCUMENT ACCORDINGLY         C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C 13JAN97 - AHH: DOCUMENT IERR=4                                       C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    int IERR;
    double TR, PR, RHOLR, RHOVR;

    TR = TK / TCW;

    IERR = PCOEX(TR, &PR, &RHOLR, &RHOVR, IWORK, PROPR);

    *PMPA = PR * PCW;
    *RHOV = RHOVR * RHOCW;
    *RHOL = RHOLR * RHOCW;

    return IERR;
}

int TSAT(double PMPA, double *TK, double *RHOL, double *RHOV, int IWORK[], double PROPR[])
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** TSAT ***                                     C
    C THIS ROUTINE COMPUTES THE COEXISTENCE TEMPERATURE (IN K) AT A        C
    C GIVEN PRESSURE (IN MPA).  IT ALSO RETURNS THE COEXISTING             C
    C DENSITIES.  IT CALLS TCOEX, WHICH WORKS IN REDUCED UNITS             C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C PMPA    R    I   PRESSURE, MPA                                       C
    C TK      R    O   COEXISTENCE TEMPERATURE, K                          C
    C RHOL    R    O   SATURATED LIQUID DENSITY, KG/M3                     C
    C RHOV    R    O   SATURATED VAPOR DENSITY, KG/M3                      C
    C IWORK   IA   -   INTEGER VECTOR FOR USE BY LOWER-LEVEL ROUTINES      C
    C PROPR   RA   -   VECTOR OF REDUCED PROPERTIES USED BY LOWER-LEVEL    C
    C                  ROUTINES                                            C
    C IERR    I    O   ERROR FLAG (FROM TCOEX)                             C
    C                  0 = SUCCESS                                         C
    C                  1 = PRESSURE OUT OF RANGE                           C
    C                  2 = UNABLE TO BRACKET (PROBABLY BECAUSE VERY CLOSE  C
    C                      TO TC) RETURN VALUES FROM SATURATION EQUATIONS  C
    C                  3 = UNABLE TO CONVERGE BOUNDED SOLUTION.  RETURN    C
    C                      LAST GUESS FOR SOLUTION                         C
    C                  4 = TEMPERATURE BELOW TRIPLE POINT BUT IN RANGE     C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 07SEP95 - INITIAL CREATION BY AHH                                    C
    C 28SEP95 - AHH: PARAMETERIZE NUMBER OF PROPERTIES                     C
    C 29SEP95 - AHH: CHANGE IWANT TO IWORK, REDOCUMENT ACCORDINGLY         C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C 13JAN97 - AHH: DOCUMENT IERR=4                                       C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    int IERR;
    double PR, TR, RHOLR, RHOVR;

    PR = PMPA / PCW;

    IERR = TCOEX(PR, &TR, &RHOLR, &RHOVR, IWORK, PROPR);

    *TK = TR * TCW;
    *RHOL = RHOLR * RHOCW;
    *RHOV = RHOVR * RHOCW;

    return IERR;
}

/*
    NOTE - HSSOLV is not called, therefore it was not converted to C and has been deleted.

    SUBROUTINE HSSOLV(MODE, TP, HS, TPOUT, D1, DV, DL, I2PH, Q, IWORK, PROPR, IERR)
*/

int TSUB(double *TK, double PMPA)
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** TSUB ***                                     C
    C THIS ROUTINE GIVES THE SUBLIMATION TEMPERATURE TK AS A FUNCTION OF   C
    C PRESSURE PMPA USING THE EQUATION OF WAGNER ET AL.  (J. PHYS.         C
    C CHEM. REF. DATA, 23, 515 (1994).  IT SOLVES BY ITERATIVELY CALLING   C
    C PSUB.                                                                C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    O   SUBLIMATION TEMPERATURE IN K                        C
    C PMPA    R    I   PRESSURE IN MPA                                     C
    C IERR    R    O   RETURN CODE:                                        C
    C                  0 = SUCCESS                                         C
    C                  1 = PRESSURE BELOW VALID RANGE (TK=190 K)           C
    C                  2 = PRESSURE ABOVE VALID RANGE (TK=273.16 K)        C
    C                  IF 1 OR 2, 190 OR 273.16 IS RETURNED                C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 27SEP95 - INITIAL CREATION BY AHH                                    C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    static const double TMAX = 273.16;
    static const double TMIN = 190.0;
    static const double TOL = 1.0e-8;

    int IERR;
    double PMIN, PMAX, FHIGH, FLOW, TAUHI, TAULOW, FRDIST, TNEW, TAUNEW, FNEW, PNEW;

    /*
    C FIRST, ESTABLISH BOUNDS AND SEE IF YOU ARE INSIDE
    */
    IERR = PSUB(TMIN, &PMIN);
    IERR = PSUB(TMAX, &PMAX);

    if (PMPA < PMIN)
    {
        *TK = TMIN;
        IERR = 1;
    }
    else if (PMPA > PMAX)
    {
        *TK = TMAX;
        IERR = 2;
    }
    else
    {
        /*
        C SOLUTION IS BOUNDED BETWEEN TMAX AND TMIN, SO DO BOUNDED SECANT
        C WITH LOG P LINEAR IN 1/T
        */
        FHIGH = log(PMAX / PMPA);
        FLOW = log(PMIN / PMPA);
        TAULOW = 1.0 / TMIN;
        TAUHI = 1.0 / TMAX;

        for (int i = 1; i <= 100; i++)
        {
            FRDIST = -FLOW / (FHIGH - FLOW);

            if (FRDIST < 0.01)
                FRDIST = 0.01;

            if (FRDIST > 0.99)
                FRDIST = 0.99;

            TAUNEW = TAULOW + FRDIST * (TAUHI - TAULOW);
            TNEW = 1.0 / TAUNEW;

            IERR = PSUB(TNEW, &PNEW);

            FNEW = log(PNEW / PMPA);
            /*
            C CHECK FOR CONVERGENCE
            */
            if (fabs(FNEW) < TOL)
            {
                /*
                C CONVERGENCE
                */
                IERR = 0;
                *TK = TNEW;
                return IERR;
            }
            /*
            C MAKE NEW BOUNDS DEPENDING ON WHETHER NEW GUESS WAS HIGH OR LOW
            */
            if (FNEW > 0.0)
            {
                FHIGH = FNEW;
                TAUHI = TAUNEW;
            }
            else
            {
                FLOW = FNEW;
                TAULOW = TAUNEW;
            }
        }

        /*
        C IF YOU GET TO HERE, YOU DID NOT CONVERGE ON THE BOUNDED ROOT
        */
        IERR = 3;
        *TK = TNEW;
        return IERR;
    }

    return IERR;
}

int PSUB(double TK, double *PMPA)
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** PSUB ***                                     C
    C THIS ROUTINE GIVES THE SUBLIMATION PRESSURE PMPA AS A FUNCTION OF    C
    C TEMPERATURE TK USING THE EQUATION OF WAGNER ET AL.  (J. PHYS.        C
    C CHEM. REF. DATA, 23, 515 (1994)                                      C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    I   TEMPERATURE IN K                                    C
    C PMPA    R    O   SUBLIMATION PRESSURE IN MPA                         C
    C IERR    R    O   RETURN CODE:                                        C
    C                  0 = SUCCESS                                         C
    C                  1 = TEMPERATURE BELOW VALID RANGE (190 K)           C
    C                  2 = TEMPERATURE ABOVE VALID RANGE (273.16 K)        C
    C                  IF 1 OR 2, AN EXTRAPOLATION IS STILL RETURNED       C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 27SEP95 - INITIAL CREATION BY AHH                                    C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    static const double TN = 273.16;
    static const double TMIN = 190.0;
    static const double PN = 611.657e-6;
    static const double A1 = -13.928169;
    static const double A2 = 34.7078238;

    int IERR;
    double THETA, FSUB;

    IERR = 0;

    if (TK < TMIN)
        IERR = 1;
    else if (TK > TN)
        IERR = 2;

    THETA = TK / TN;
    FSUB = A1 * (1.0 - pow(THETA, -1.5)) + A2 * (1.0 - pow(THETA, -1.25));
    *PMPA = exp(FSUB) * PN;

    return IERR;
}

int TMELT(double *TK, double PMPA, int *IFORM)
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** TMELT ***                                    C
    C THIS ROUTINE GIVES THE MELTING TEMPERATURE TK AS A FUNCTION OF       C
    C PRESSURE PMPA USING THE EQUATIONS OF WAGNER ET AL.  (J. PHYS.        C
    C CHEM. REF. DATA, 23, 515 (1994).  IT SOLVES BY ITERATIVELY CALLING   C
    C PMELT1 (FOR ICE I) OR PMELT2 (FOR HIGH-PRESSURE FORMS).              C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    O   MELTING TEMPERATURE IN K                            C
    C PMPA    R    I   PRESSURE IN MPA                                     C
    C IFORM   I    O   NUMBER OF ICE FORM INVOLVED IN THE EQUILIBRIUM      C
    C IERR    I    O   RETURN CODE:                                        C
    C                  0 = SUCCESS                                         C
    C                  1 = PRESSURE BELOW VALID RANGE (611.657D-6 MPA)     C
    C                  2 = PRESSURE ABOVE VALID RANGE (2216 MPA)           C
    C                  IF 1 OR 2, TMIN OR TMAX IS RETURNED                 C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 29SEP95 - INITIAL CREATION BY AHH                                    C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    static const double PMAX = 20618.0;
    static const double PMIN = 611.657e-6;
    static const double TTOP = 715.0;
    static const double TBOT = 273.16;
    static const double T13 = 251.165;
    static const double P13 = 209.9;
    static const double TOL = 1.0e-8;

    int LHIGHP; /* in Fortran, this was a logical variable, as these don't exist in C, it
                    will be an integer where the value of 0 equates to false, all non-zero
                    values equate to true, however, a value of 1 will be used for true. */
    int IERR, IDUM;
    double THIGH, TLOW, PTOP, FHIGH, FLOW, PLOW, FRDIST, TNEW, FNEW, PNEW;

    /*
    C FIRST, ESTABLISH BOUNDS AND SEE IF YOU ARE INSIDE
    */
    if (PMPA < PMIN)
    {
        *TK = TBOT;
        IERR = 1;
    }
    else if (PMPA > PMAX)
    {
        *TK = TTOP;
        IERR = 2;
    }
    else
    {
        /*
        C ESTABLISH WHETHER YOU ARE ON THE ICE-I BRANCH OR THE HIGH-P PART
        */
        if (PMPA > P13)
        {
            LHIGHP = 1;
            THIGH = TTOP;
            TLOW = T13;
            IERR = PMELT2(TTOP, &PTOP, &IDUM);
            FHIGH = log(PTOP / PMPA);
            FLOW = log(P13 / PMPA);
        }
        else
        {
            LHIGHP = 0;
            *IFORM = 1;
            THIGH = TBOT;
            TLOW = T13;
            IERR = PMELT1(TLOW, &PLOW);
            FHIGH = -log(PMIN / PMPA);
            FLOW = -log(PLOW / PMPA);
        }
        /*
        C SOLUTION IS BOUNDED BETWEEN TMAX AND TMIN, SO DO BOUNDED SECANT
        C WITH LOG P
        */
        for (int i = 1; i <= 100; i++)
        {
            FRDIST = -FLOW / (FHIGH - FLOW);

            if (FRDIST < 0.01)
                FRDIST = 0.01;

            if (FRDIST > 0.99)
                FRDIST = 0.99;

            TNEW = TLOW + FRDIST * (THIGH - TLOW);

            if (LHIGHP)
            {
                IERR = PMELT2(TNEW, &PNEW, IFORM);
                FNEW = log(PNEW / PMPA);
            }
            else
            {
                IERR = PMELT1(TNEW, &PNEW);
                FNEW = -log(PNEW / PMPA);
            }
            /*
            C CHECK FOR CONVERGENCE
            */
            if (fabs(FNEW) < TOL)
            {
                /*
                C CONVERGENCE
                */
                IERR = 0;
                *TK = TNEW;
                return IERR;
            }
            /*
            C MAKE NEW BOUNDS DEPENDING ON WHETHER NEW GUESS WAS HIGH OR LOW
            */
            if (FNEW > 0.0)
            {
                FHIGH = FNEW;
                THIGH = TNEW;
            }
            else
            {
                FLOW = FNEW;
                TLOW = TNEW;
            }
        }

        /*
        C IF YOU GET TO HERE, YOU DID NOT CONVERGE ON THE BOUNDED ROOT
        */
        IERR = 3;
        *TK = TNEW;
        return IERR;
    }

    return IERR;
}

int PMELT(double TK, int *NROOTS, double *PMPA1, double *PMPA2, int *IFORM)
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** PMELT ***                                    C
    C THIS ROUTINE GIVES THE MELTING PRESSURE(S) PMPA AS A FUNCTION OF     C
    C TEMPERATURE TK USING THE EQUATIONS OF WAGNER ET AL.  (J. PHYS.       C
    C CHEM. REF. DATA, 23, 515 (1994).  ALL FORMS OF ICE ARE CONSIDERED,   C
    C SO IT IS POSSIBLE TO HAVE TWO ROOTS.                                 C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    I   TEMPERATURE IN K                                    C
    C NROOTS  I    O   NUMBER OF PRESSURES AT WHICH MELTING HAPPENS AT TK  C
    C PMPA1   R    O   MELTING PRESSURE IN MPA FOR FIRST ROOT, IF ANY      C
    C PMPA2   R    O   MELTING PRESSURE IN MPA FOR SECOND ROOT, IF ANY     C
    C                  IF 2 ROOTS, PMPA1 HAS LOWEST PRESSURE (ICE I)       C
    C IFORM   I    O   FORM OF ICE WITH WHICH FLUID COEXISTS.  IF NROOTS   C
    C                  IS 2, THIS CONTAINS THE HIGH-PRESSURE FORM SINCE    C
    C                  ROOT 1 MUST BE ICE I.                               C
    C IERR    I    O   RETURN CODE:                                        C
    C                  0 = SUCCESS                                         C
    C                  1 = TEMPERATURE BELOW VALID RANGE (251.165 K)       C
    C                  2 = TEMPERATURE ABOVE VALID RANGE (715. K)          C
    C                  IF 1 OR 2, PMPA IS RETURNED AS ZERO                 C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 28SEP95 - INITIAL CREATION BY AHH                                    C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    static const double TMAX = 715.0;
    static const double TMIN = 251.165;
    static const double T1MAX = 273.16;

    int IERR, IDUM;

    *PMPA1 = 0.0;
    *PMPA2 = 0.0;
    *IFORM = 0;
    *NROOTS = 0;
    IERR = 0;

    if (TK < TMIN)
        IERR = 1;

    else if (TK > TMAX)
        IERR = 2;

    else if (TK <= T1MAX)
    {
        /*
        C BOTH AN ICE-I ROOT AND A ROOT FOR A HIGHER-P FORM
        */
        IDUM = PMELT1(TK, PMPA1);
        IDUM = PMELT2(TK, PMPA2, IFORM);
        *NROOTS = 2;
    }
    else
    {
        /*
        C ONLY ROOT IS A HIGHER-PRESSURE FORM OF ICE
        */
        IDUM = PMELT2(TK, PMPA1, IFORM);
        *NROOTS = 1;
    }

    return IERR;
}

int SURF(double TK, double *SURFT)
{
    /*
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    C                     *** SURF ***                                     C
    C THIS ROUTINE COMPUTES THE VAPOR/LIQUID SURFACE TENSION               C
    C OF WATER AT A GIVEN TEMPERATURE AND AND SATURATION CONDITIONS        C
    C ACCORDING TO THE SEPTEMBER 1994 IAPWS RELEASE ON THE SURFACE TENSION C
    C OF ORDINARY WATER SUBSTANCE                                          C
    C                                                                      C
    C ARGUMENT LIST:                                                       C
    C  NAME  TYPE I/O? EXPLANATION                                         C
    C ------ ---- ---- --------------------------------------------------  C
    C TK      R    I   TEMPERATURE IN KELVINS                              C
    C SURFT   R    O   SURFACE TENSION IN N/M                              C
    C IERR    I    O   RETURN CODE:                                        C
    C                  0 = SUCCESS                                         C
    C                  1 = TEMPERATURE BELOW LOWER LIMIT FOR SATURATION    C
    C                      CALCULATION, ZERO RETURNED                      C
    C                  2 = TEMPERATURE ABOVE CRITICAL POINT, ZERO RETURNED C
    C                  3 = 2-PHASE REGION EXTRAPOLATED BELOW TRIPLE POINT  C
    C                                                                      C
    C MAINTENANCE:                                                         C
    C                                                                      C
    C 19OCT95 - INITIAL IMPLEMENTATION BY AHH                              C
    C 30AUG96 - AHH: LOWERCASE INCLUDES FOR OTHER PLATFORMS                C
    C 27MAY97 - AHH: IMPLEMENT IERR=3                                      C
    C                                                                      C
    CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    */
    int IERR;
    double TAU;

    if (TK < TSMIN)
    {
        IERR = 1;
        *SURFT = 0.0;
    }
    else if (TK > TCW)
    {
        IERR = 2;
        *SURFT = 0.0;
    }
    else
    {
        if (TK < TTRIPW)
            IERR = 3;
        else
            IERR = 0;

        TAU = 1.0 - TK / TCW;
        *SURFT = 0.2358 * pow(TAU, 1.256) * (1.0 - 0.625 * TAU);
    }

    return IERR;
}

/*
  NOTE - PROPS0 is not called and so has been deleted from the C file.

      SUBROUTINE PROPS0(IWANT, TK, RHO, PROPSI, PROPR, I2PHCK, I2PH,
     >                  ISCHK, ISFLG, ICCHK, ICFLG, IPCOLD, IPFOLD)
*/
